"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeResolverCacheFromLockfileAsync = void 0;
const helpers_1 = require("./helpers");
function isPackageCompatible(pack, platformInfo) {
    var _a, _b, _c;
    if ((_a = pack.os) === null || _a === void 0 ? void 0 : _a.every((value) => value.toLowerCase() !== platformInfo.os)) {
        return false;
    }
    if ((_b = pack.cpu) === null || _b === void 0 ? void 0 : _b.every((value) => value.toLowerCase() !== platformInfo.cpu)) {
        return false;
    }
    if ((_c = pack.libc) === null || _c === void 0 ? void 0 : _c.every((value) => value.toLowerCase() !== platformInfo.libc)) {
        return false;
    }
    return true;
}
/**
 * Given a lockfile and information about the workspace and platform, computes the resolver cache file.
 * @param params - The options for computing the resolver cache
 * @returns A promise that resolves with the resolver cache file
 */
async function computeResolverCacheFromLockfileAsync(params) {
    var _a;
    const { workspaceRoot, commonPrefixToTrim, platformInfo, projectByImporterPath, lockfile, afterExternalPackagesAsync } = params;
    const contexts = new Map();
    const missingOptionalDependencies = new Set();
    // Enumerate external dependencies first, to simplify looping over them for store data
    for (const [key, pack] of lockfile.packages) {
        let name = pack.name;
        const descriptionFileRoot = (0, helpers_1.getDescriptionFileRootFromKey)(workspaceRoot, key, name);
        // Skip optional dependencies that are incompatible with the current environment
        if (pack.optional && !isPackageCompatible(pack, platformInfo)) {
            missingOptionalDependencies.add(descriptionFileRoot);
            continue;
        }
        const integrity = (_a = pack.resolution) === null || _a === void 0 ? void 0 : _a.integrity;
        if (!name && key.startsWith('/')) {
            const versionIndex = key.indexOf('@', 2);
            name = key.slice(1, versionIndex);
        }
        if (!name) {
            throw new Error(`Missing name for ${key}`);
        }
        const context = {
            descriptionFileRoot,
            descriptionFileHash: integrity,
            isProject: false,
            name,
            deps: new Map(),
            ordinal: contexts.size,
            optional: pack.optional
        };
        contexts.set(descriptionFileRoot, context);
        if (pack.dependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, pack.dependencies, context);
        }
        if (pack.optionalDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, pack.optionalDependencies, context);
        }
    }
    if (afterExternalPackagesAsync) {
        await afterExternalPackagesAsync(contexts, missingOptionalDependencies);
    }
    // Add the data for workspace projects
    for (const [importerPath, importer] of lockfile.importers) {
        // Ignore the root project. This plugin assumes you don't have one.
        // A non-empty root project results in global dependency hoisting, and that's bad for strictness.
        if (importerPath === '.') {
            continue;
        }
        const project = projectByImporterPath.findChildPath(importerPath);
        if (!project) {
            throw new Error(`Missing project for importer ${importerPath}`);
        }
        const context = {
            descriptionFileRoot: project.projectFolder,
            descriptionFileHash: undefined, // Not needed anymore
            name: project.packageJson.name,
            isProject: true,
            deps: new Map(),
            ordinal: contexts.size
        };
        contexts.set(project.projectFolder, context);
        if (importer.dependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.dependencies, context);
        }
        if (importer.devDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.devDependencies, context);
        }
        if (importer.optionalDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.optionalDependencies, context);
        }
    }
    // Convert the intermediate representation to the final cache file
    const serializedContexts = Array.from(contexts, (0, helpers_1.createContextSerializer)(missingOptionalDependencies, contexts, commonPrefixToTrim));
    const cacheFile = {
        contexts: serializedContexts
    };
    return cacheFile;
}
exports.computeResolverCacheFromLockfileAsync = computeResolverCacheFromLockfileAsync;
//# sourceMappingURL=computeResolverCacheFromLockfileAsync.js.map